/*=============================================================================
author        :Walter Schreppers & Nhan Truong
filename      :HTTPServer.java
created       :20/12/2000
modified      :27/12/2000
version       :2
bugreport(log):/
=============================================================================*/
import java.io.*;
import java.net.*;

public class HTTPServer extends Thread {

	public static final int DEFAULT_PORT = 80;
	public static final int CONNECTION_MAX = 10;
	private static final String USAGE =
		"USAGE: java HTTPServer [-p <port_nr>] [-m <max_conn>] [-f]\n" +
		"options:\n" + 
			"\tport_nr: port number the server will be listening to\n" + 
			"\tmax_conn: maximum number of simultaneous connections allowed" +
			"\t-w: do not create a window";

	private ServerSocket server = null;
	private boolean stop = false;
	
	private int port = DEFAULT_PORT;
	private int maxcon = CONNECTION_MAX;

	public HTTPServer()
	throws IOException {
		server = new ServerSocket(DEFAULT_PORT, CONNECTION_MAX);
	}
	
	public HTTPServer(int port)
	throws IOException, IllegalArgumentException {
		this.port = port;
		server = new ServerSocket(port, CONNECTION_MAX);
	}
	
	public HTTPServer(int port, int maxcon)
	throws IOException {
		this.port = port;
		this.maxcon = maxcon;
		server = new ServerSocket(port, maxcon);
	}
	
	public int getPort() {
		return port;
	}
	
	public int getMaxConnections() {
		return maxcon;
	}
	
	/* The server is a thread, so this run method contains the main code of
	 * the server.  It keeps on listening (while loop) on the specified port for a certain
	 * interval.  After this period of time has passed, it takes a timeout to
	 * check if the user has stopped the server.  If so, this run method wil
	 * stop, and eventually the server will be stopped.  If not, the while loop persists
	 * and the server listen again.
	 */
	public void run() {
		try {
			/* set server timeout (1000 milliseconds).  After this period of time
			 * the server will check the exit status.
			 */
			server.setSoTimeout(1000);
			Output.out.println("WN HTTPServer\n" +
				"Copyright (c) 2000-2001\n" +
				"Walter Schreppers & Nhan Truong\n" +
				"Starting Server on port " + port + " ...\n");
			/* In this while-loop the server keeps on listening on the
			 * port for new connection requests.
			 * when receiving a new incoming request, server.accept(), a new
			 * 'Connection' object is created.  this object is a thread that
			 * that handles a new connection, and starts itself in 
			 * its constructor.
			 * After the specified timeout, an InterruptedIOExcption will be thrown,
			 * and the server will check the exit status, ie. if the user has requested
			 * to stop the server. (This only works if the server is running in a GUI window,
			 * ie. when the user has pressed the stop button).
			 */
			while(true){
				try {
					Connection connect = new Connection(server.accept());	// incoming request
				}
				catch (InterruptedIOException ie) {	// timeout
					if (stop) {		// check exit status (ie. should server be stopped?)
						Output.out.print("Going offline ... ");
						server.close();
						server = null;
						Output.out.println("Server stopped.\n");
						return;
					}
				}
			}
		}
		catch (Exception e) {
			Output.err.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
	/* This method is meant to stop the server.  All it does is setting the boolean
	 * value 'stop' to true. The server will continously check this boolean exit status
	 * after a specified interval.
	 */
	public void exit() {
		stop = true;
	}
	
	/* the main method for starting the server.
	 */
	public static void main(String args[]) {
		Options opts = parseCommandLine(args);
		
		if (opts.createFrame) {		// run in a GUI window
			ServerWindow serverwin = new ServerWindow(opts.port);	// create new server window
			Output.out.area = serverwin.getOutputArea();			// set output stream
			Output.err.area = serverwin.getErrorOutputArea();		// set error stream
			Output.mode = Output.WIN;		// set output mode: WINDOW
			serverwin.show();
		} else {		// run from command line
			try {
				HTTPServer s = new HTTPServer(opts.port, opts.maxcon);	// create new HTTPServer object
				Output.mode = Output.CONSOLE;	// set output mode: CONSOLE
				s.start();						// start the server
			} catch (IOException e) {
				System.err.println(e.getMessage());
			}
		}
	}
	
	/* This method parses the command line.  It takes an array of strings (the
	 * command line arguments).  And returns the user specified options, as
	 * an 'Options' object.
	 */
	public static Options parseCommandLine(String args[]) {
		Options opts = new Options(DEFAULT_PORT, CONNECTION_MAX);
		
		String arg = null;
		for (int i = 0; i < args.length; i++) {
			arg = args[i++];
			try {
				if (arg.equals("-p")) {
					opts.port = Integer.parseInt(args[i]);
				} else if (arg.equals("-m")) {
					opts.maxcon = Integer.parseInt(args[i]);
				} else if (arg.equals("-w")) {
					opts.createFrame = false;
				} else if (arg.startsWith("-")) {
					throw new Exception("unknown option: " + arg);
				} else {
					throw new Exception("");
				}
			} catch (Exception e) {
				System.out.println(e.getMessage());
				System.out.print(USAGE);
				System.exit(0);
			}
		}
		
		return opts;
	}
}

/*
 * Class Options holds some public fields that represents some options
 * the user can specify at the command line, when starting the server.
 * The options are:
 *	- 'port':			the port number the server will be listening to
 *	- 'maxcon':			the maximum number of simultaneous connections allowed
 *	- 'rootdir': 		the root directory (not used)
 *	- 'createFrame':	boolean value.  If true, the server will be running
 *						in a GUI window, instead of from the command line
 */
class Options {
	
	public int port;
	public int maxcon;
	public String rootdir = null;
	public boolean createFrame = true;
	
	public Options(int port, int maxcon) {
		this.port = port;
		this.maxcon = maxcon;
	}
	
}

