/*=============================================================================
author        :Walter Schreppers & Nhan Truong
filename      :JspParser.java
created       :20/12/2000
modified      :30/01/2001
version       :2
bugreport(log):/
=============================================================================*/
import java.io.*;

/* This class parses a single JSP file, and converts it to a Java class file.
 * Following tags are parsed:
 *	- import directive:	<% page import="..." %>
 *	- comments:		everything between <%-- ... --%>
 *	- expressions:	everything between <%= ... %>
 *	- scriplets:	everthing between <% ... %>
 *	- everything else is supposed to be template data.
 */

public class JspParser {
	
	private static final String JSP_HEAD =
		"/*\r\n * This is a generated java file.\r\n" +
		" * Please do not edit!\r\n */\r\n\r\n" +
		"import java.io.*;\r\n" +
		"import java.net.*;\r\n";
		
		
	private String name = null;
	private String line = null;
	private int sp = 0;
	private BufferedReader source = null;
	private PrintWriter target = null;
	
	public JspParser(String name, FileReader source, FileWriter target)
	throws IOException {
		this.name = name;
		this.source = new BufferedReader(source);
		this.target = new PrintWriter(new BufferedWriter(target));
		this.line = this.source.readLine();
	}
	
	public void parse()
	throws IOException {
		String javaSource=new String();
		javaSource=writeClassBegin();

		while (readLine() != null) {
			javaSource+=template()+"\n";
			if (line == null) break;
			if (line.startsWith("<%--")) {
				skip();				// skip comments
			} else if (line.startsWith("<%=")) {
				line = line.substring(3);
				javaSource+=expression()+"\n";	// get expression
			} else if (line.startsWith("<%@")){
				line= line.substring(3);
				javaSource=directive()+javaSource+"\n";
			} else if (line.startsWith("<%")) {
				line = line.substring(2);
				javaSource+=scriplet()+"\n";		// get script code (java code)
			} 
		}

		javaSource+=writeClassEnd();

		javaSource=JSP_HEAD+javaSource;

		target.println(javaSource);
		source.close();
		target.close();
	}
	
	private String readLine()
	throws IOException {
		if ( (line != null) && (line.length() == 0))
			line = source.readLine();
		return line;	
	}
	
	private String template()
	throws IOException {
		int index = -1;
		String tstring = null;
		if ((index = line.indexOf("<%")) >= 0) {
			tstring = line.substring(0, index);
			line = line.substring(index);
		} else {
			tstring = line;
			line = source.readLine();
		}
		return "out.print(\"" + filterTemplate(tstring) + "\");";
	}
	
	private void skip()
	throws IOException {
		int index = -1;
		do {
			if ((index = line.indexOf("--%>")) >= 0) {
				line = line.substring(index + 4);
				return;
			}
		} while ((line = source.readLine()) != null);
		System.out.println("end skip...");
	}
	
	private String scriplet()
	throws IOException {
		String script = new String();
		int index = -1;
		do {
			if ((index = line.indexOf("%>")) >= 0) {
				script += line.substring(0, index);
				line = line.substring(index + 2);
				break;
			}
			script += line+"\n";
		} while ((line = source.readLine()) != null);
		return script;
	}
	private String directive()
	throws IOException{
		String directive=new String();
		expandLine();
		skipWhiteSpace();
		if(line.indexOf("page")!=0) return ""; //'page' must be found 
		line=line.substring(4);
		skipWhiteSpace();
		
		if(line.indexOf("import")!=0){
			Output.err.println("Warning only import directive is supported in page "+name);
			return ""; //'import' is only directive supported
		}
		line=line.substring(6);
		skipWhiteSpace();
		if(line.indexOf("=")!=0) return "";
		line=line.substring(1);
		skipWhiteSpace();
		if(line.indexOf("\"")!=0) return "";
		line=line.substring(1);
		if(line.indexOf("\"")<0) return ""; //closing "
		String includes=line.substring(0,line.indexOf("\""));
		includes = "import " + includes;
		while(includes.indexOf(",")>=0){
			includes = stringReplace(includes,",",";\nimport ");
		}
		includes = includes + ";\n";
		skipWhiteSpace();
		if(line.indexOf("%>")<0) return "";
		line=line.substring(line.indexOf("%>")+2);
		return includes;
	}

	private void expandLine()
	throws IOException{
		//for the case that the import directive is more than 1 line
		int index = -1;
		String temp=new String();
		do {
			line+=temp;
			if ((index = line.indexOf("%>")) >= 0) {
				break;
			}
			temp=source.readLine();
		} while (temp != null);
	}
	
	private String stringReplace(String source,String find,String repl){
		int start=source.indexOf(find);
		int end=start+find.length();
		StringBuffer temp=new StringBuffer(source);
		temp.replace(start,end,repl);
		return temp.toString();
	}	

	private void skipWhiteSpace()
	throws IOException{
		int i=0;
		while((line.charAt(i)==' ')||(line.charAt(i)=='\n')) i++;
		line=line.substring(i);
	}
	


	private String expression()
	throws IOException {
		return "out.print(" + scriplet() + ");";
	}
	
	private String writeClassBegin(){
		return 	" public class "+name+" extends JSPBase{   	\n"+
				"	public "+name+"(){                  	\n"+
				"	}                                 		\n"+
				"	public void service(Request request, Response response){\n"+
				"	PrintStream out = response.getOutputStream();\n"+
				"		//the jsp script code \n";
	}
	private String writeClassEnd(){
		return 	"		//end script code	\n"+
				"	}	\n"+
				"}	\n";
	}
	
	private String filterTemplate(String input) {
		String output = new String();
		for (int index = 0; index < input.length(); index++) {
			switch (input.charAt(index)) {
				case '"': case'\'':
					output += '\\';
					break;
			}
			output += input.charAt(index);
		}
		
		return output;
	}
	
}
